#ifndef _FSUTILS_H
#define _FSUTILS_H

#include <stdbool.h>
#include "ff.h"

enum find_returnValues {
   FIND_NOT_FOUND = -1,
   FIND_FOUND,
   /* Remaining return values are FRESULT error definitions */
};


/**
 * \brief Look for a file or a directory in a directory
 *
 * This function looks for a file or a directory in the given directory,
 * optionally searching sub directories. This function utilises recursion, and
 * should be used with care in deep file systems. If fileAttr is non-NULL, the
 * file's paramters will be stored if the file was found. In order to search
 * for directories, fileAttr must be non-NULL and contain AM_DIR when passed
 * to the function.
 *
 * \param searchDir the directory to search in
 * \param fileName the name of the file or directory to search for
 * \param searchSubDirs if true, search sub directories
 * \param fileAttr pointer to a byte, which if non-NULL will be filled with
 * the found file's attributes
 *
 * \return FIND_NOT_FOUND, FIND_FOUND or any of the defined FRESULT error
 * values
 */
int find( const char searchDir[], const char fileName[], bool searchSubDirs,
BYTE *fileAttr );

#endif // _FSUTILS_H
