/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MCUC_H
#define MCUC_H

#include <stdbool.h>
#include "packet.h"
#include "spi.h"

union packet mcuc_makeStatusPacket();

int mcuc_handleReceivedPacket( int packetType, uint16_t *packetData,
      uint32_t packetLen );

void mcuc_handleError( struct errorInfo error );

void mcuc_handleStatus( union packet packet );

/**
 * \brief Prepare a response for an incoming query
 *
 * Call mcuc_sendPacket() to load response into the module for transmission as
 * when mcuc_doSPIWork() is called. If no response is loaded, an internal
 * error will be sent instead.
 *
 * \param packetType the type of query
 */
void mcuc_prepareQueryResponse( int packetType, uint16_t *packetData,
      uint32_t packetLen );


void mcuc_init();

void mcuc_doSPIWork();

void mcuc_sendPacket( const union packet *packet );

bool mcuc_busy();

#endif // MCUC_H
