/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MCUC_PROTOCOL_H
#define MCUC_PROTOCOL_H

#include <stdint.h>

struct __attribute__(( packed )) errorInfo
{
   uint16_t               : 2;
   uint16_t abort         : 1;
   uint16_t error         : 13;
};

/**
 * \brief Error types used in errorInfo::error
 */
enum errorInfoTypes
{
   Error_unknown = 0,
   Error_SPIOverflow,
   Error_internal,
   Error_servingOtherQuery,
   Error_malloc,
   Error_packetFormat,
   Error_notReady,
};

/**
 * \brief Status packet for PIC32
 */
struct __attribute__(( packed )) p32StatusPacket
{
   uint16_t                     : 2;
   uint16_t timeUpdate          : 1;
   uint16_t dateUpdate          : 1;
   uint16_t availabilityUpdate  : 1;
   uint16_t online              : 1;
   uint16_t memCardPresent      : 1;
   uint16_t otherUpdate         : 1;
   uint16_t ready               : 1;
   uint16_t RESERVED            : 8;
};

/**
 * \brief Status packet for PIC24
 */
struct __attribute__(( packed )) p24StatusPacket
{
   uint16_t                     : 2;
   uint16_t RESERVED            : 14;
};

#define PackType_status 2
#define MCUC_USER_FORMATS \
   struct p32StatusPacket p32Status; \
   struct p24StatusPacket p24Status;

enum queryType
{
   Type_INVALID = 0,

   Type_invalidateLast,
   Type_abort,
   Type_errorString,

   /**
    * \brief Poke the other device
    *
    * Sends a 16-bit unsigned integer and expects the same integer plus one in
    * return.
    */
   Type_poke,
   Type_name,
   Type_title,
   Type_portrait,
   Type_availability,
   Type_availDuration,
   Type_time,
   Type_date,
};

#endif // MCUC_PROTOCOL_H
