/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MCUC_SPI_H
#define MCUC_SPI_H

#if defined __PIC24F__
#include "peripheralLibConfig.h"
#include <PPS.h>
#include <spi.h>
#elif defined __PIC32MX__
#include "HardwareProfile.h"
#include <plib.h>
#endif

#if defined __PIC24F__
#define MCUC_master
#define spi_overflow() SPI1STATbits.SPIROV
#define spi_clearOverflow() ( SPI1STATbits.SPIROV = 0 )

#define spi_recvBufEmpty() !SPI1STATbits.SPIRBF
#define spi_write( d ) do { while ( SPI1STATbits.SPITBF ); SPI1BUF = d; } \
   while( 0 )
#define spi_read() SPI1BUF

#elif defined __PIC32MX__
#define MCUC_slave
#define spi_overflow() SPI3STATbits.SPIROV
#define spi_clearOverflow() ( SPI3STATCLR = _SPI3STAT_SPIROV_MASK )

#define spi_recvBufEmpty() !SPI3STATbits.SPIRBF
#define spi_write( d ) do { while ( !SPI3STATbits.SPITBE ); SPI3BUF = d; } \
   while( 0 )
#define spi_read() SPI3BUF

#else
#error "PIC is neither PIC24F nor PIC32MX"
#endif

/**
 * \brief Initialise the SPI module
 */
void spi_init();

#endif // MCUC_SPI_H
