/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \file
 * \brief Hardware configuration file for Microchip application libraries
 *
 * \author Andreas Misje
 * \date 21.02.13
 */

#ifndef HARDWARE_PROFILE_H
#define HARDWARE_PROFILE_H

//#include <Compiler.h>

/* 8 MHz * 4 PLL = 32 MHz: */
#define GetSystemClock()                     32000000UL
#define GetPeripheralClock()                 ( GetSystemClock() / 2 )
#define GetInstructionClock()                ( GetSystemClock() / 2 )
#define FCY GetInstructionClock()

/* Configure graphics library: */
#define GFX_USE_DISPLAY_CONTROLLER_MCHP_DA210
#define USE_16BIT_PMP
#define GFX_GCLK_DIVIDER                     80
#define GFX_EPMP_CS1_BASE_ADDRESS            0x20000ul
#define GFX_DISPLAY_BUFFER_START_ADDRESS     0x20000ul
/* 480 x 272 * 16 / 8 * 2 = 522,240 = 0x7f800 (double buffering): */
#define GFX_DISPLAY_BUFFER_LENGTH            0x7f800ul
/* 1 MB (16-bit) SRAM: */
#define GFX_EPMP_CS1_MEMORY_SIZE             0x100000ul

#define USE_GFX_EPMP

/* EPMP timing and polarity configuration: */
#define EPMPCS1_CS_POLARITY                  GFX_ACTIVE_LOW
#define EPMPCS1_WR_POLARITY                  GFX_ACTIVE_LOW
#define EPMPCS1_RD_POLARITY                  GFX_ACTIVE_LOW
#define EPMPCS1_BE_POLARITY                  GFX_ACTIVE_LOW
#define EPMPCS1_ACCESS_TIME                  55

#define GFX_DISPLAYENABLE_ENABLE
#define GFX_HSYNC_ENABLE
#define GFX_VSYNC_ENABLE
#define GFX_DISPLAYPOWER_ENABLE
#define GFX_CLOCK_POLARITY                   GFX_ACTIVE_LOW
#define GFX_DISPLAYENABLE_POLARITY           GFX_ACTIVE_HIGH
#define GFX_HSYNC_POLARITY                   GFX_ACTIVE_LOW
#define GFX_VSYNC_POLARITY                   GFX_ACTIVE_LOW
#define GFX_DISPLAYPOWER_POLARITY            GFX_ACTIVE_HIGH

/* Configure glass (NHD-4.3-480272EF-ATXL-CTP): */
#define DISP_ORIENTATION                     0
#define DISP_HOR_RESOLUTION                  480
#define DISP_VER_RESOLUTION                  272
#define DISP_DATA_WIDTH                      24
#define DISP_HOR_PULSE_WIDTH                 41
#define DISP_HOR_BACK_PORCH                  2
#define DISP_HOR_FRONT_PORCH                 2
#define DISP_VER_PULSE_WIDTH                 10
#define DISP_VER_BACK_PORCH                  2
#define DISP_VER_FRONT_PORCH                 2
#define GFX_LCD_TYPE                         GFX_LCD_TFT

#define BACKLIGHT_ENABLE_LEVEL               1
#define BACKLIGHT_DISABLE_LEVEL              0

/* Definitions for POWER ON pin: */
#define DisplayPowerConfig()
#define DisplayPowerOn()
#define DisplayPowerOff()

/* Definitions for backlight control pin: */
#define DisplayBacklightConfig()        
#define DisplayBacklightOn()        
#define DisplayBacklightOff()          

/* External flash configuration: */
#define GFX_EPMP_CS2_BASE_ADDRESS ( GFX_EPMP_CS1_BASE_ADDRESS + \
      GFX_EPMP_CS1_MEMORY_SIZE / 2 )
#define GFX_EPMP_CS2_MEMORY_SIZE 0x200000ul
#define EPMPCS2_CS_POLARITY   GFX_ACTIVE_LOW
#define EPMPCS2_WR_POLARITY   GFX_ACTIVE_LOW
#define EPMPCS2_RD_POLARITY   GFX_ACTIVE_LOW
#define EPMPCS2_BE_POLARITY   GFX_ACTIVE_LOW
#define EPMPCS2_ACCESS_TIME   70

#endif // HARDWARE_PROFILE_H
