/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * \defgroup I2CDriver I²C driver
 *
 * \author Andreas Misje
 * \date 21.02.13
 *
 * \brief A simple I²C driver
 *
 * This is yet another I²C driver. It was written because Microchip's I²C
 * driver left most of the low-level housekeeping to the user. This driver
 * attempts to eliminate as much busy-waiting as possible. A state machine is
 * implemented in the I²C interrupt routine. Data transmission and reception
 * is performed in this state machine using a shared TX/RX data buffer.
 *
 * \anchor i2cAddrNote \note All slave device addresses will be left
 * bit-shifted inside this driver and the eight read/write bit will be
 * set/unset automatically. Please provide an unshifted address.
 */

/**
 * \defgroup TickTimer Tick timer
 *
 * \author Andreas Misje
 * \date 15.03.13
 *
 * \brief Provides a tick timer calling provided callbacks at given intervals
 *
 * Tick timer occupies a 16-bit hardware timer and uses it to update a tick
 * value. The interval between every tick update is set by the user, but it
 * must be between 1 and 1000 ms. The accuracy of the ticks depend on the
 * interval chosen by the user, the timer priority set and whether the timer
 * interrupt routine runs undisturbed. 
 *
 * In addition to provide a tick counter, this module enables the user to
 * register callback functions that will be fired at given intervals (or once
 * after a certain delay). This functionality can help eliminating any
 * busy-while delay loops, but its accuracy is limited to how often
 * tickTimer_handleCallbacks() gets called in the main loop.
 * tickTimer_handleCallbacks() iterates over an array of callbacks and fires
 * them in their registered order if their timestamp has expired.
 */

/**
 * \defgroup ft5x06Driver FT5x06 driver
 *
 * \author Andreas Misje
 * \date 21.02.13
 *
 * \brief A simple driver for the FT5x06 touch controller using I²C
 *
 * Implementation is based on the \ref I2CDriver "I²C driver" and requires the
 * user to query for touch data, wait until the I²C driver has finished
 * receiving and then retrieve the data from the I²C RX buffer. Touch data
 * should only be queried when the FT5x06 controller has sent an interrupt.
 */

/**
 * \defgroup touchScreen Touch screen interface 
 *
 * \author Andreas Misje
 * \date 18.03.13
 *
 * \brief Interface between a touch screen driver and Microchip's Graphic
 * Object Layer
 *
 * This module takes care of querying a touch screen driver for touch data at
 * appropriate times (e.g. at a defined interval using a timer interrupt or by
 * interrupts from an interrupt wire).
 */

/**
 * \defgroup sst39FlashDriver SST39 flash memory driver
 *
 * \author Andreas Misje
 * \date 20.03.13
 *
 * \brief Driver for SST39 series parallell flash memory devices
 */

/**
 * \defgroup stateMachine State machine
 *
 * \author Andreas Misje
 * \date 27.03.13
 *
 * \brief Finite state machine
 *
 * A finite state machine implementation that supports nested states, guard
 * conditions and entry/exit routines. No static variables are used, making it
 * possible to run numerous instances of the state machine. No heap allocation
 * is performed; all data is given as pointers to the module and can be stored
 * either on the stack or the heap.
 */

/**
 * \defgroup uartDriver UART driver
 *
 * \author Andreas Misje
 * \date 29.03.13
 *
 * \brief UART driver with a simple input parser
 *
 * This module performs a primitive hard-coded UART initialisation and
 * optionally supports a string parser for received UART data. For every
 * received newline, input is compared against a list of recognised commands.
 * If a command is recognised, a callback is executed. The callback is given
 * the sequence of characters between the recognised command string and
 * terminating newline as an argument.
 */
