/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "config/PICConfig.h"
#include <GenericTypeDefs.h>
#include "peripheralLibConfig.h"
#include <PPS.h>
#include "uartDriver.h"
#include <stdio.h>
#include <stdbool.h>
#include </opt/microchip/xc16/current/support/peripheral_24F/timer.h>
#include "i2cDriver.h"
#include "ft5x06.h"
#include <Graphics/GOL.h>
#include <Graphics/DisplayDriver.h>
#include "tickTimer.h"
#include "touchScreen.h"

#include <stdio.h>
#include "GOLSchemes.h"
#include "keyboard.h"
#include "m29w320FlashDriver.h"
#include <string.h>
#include <libpic30.h>
#include "mcuC/mcuc.h"

static void stillAliveTicker( void *dummy );
static void flashErase( const char *dummy );
static void flashReadTest( const char *dummy );
static void flashWriteTest( const char *dummy );
static void spiTest( const char *string );
static void spi( void *dummy )
{
   mcuc_doSPIWork();
}
static void mcucStatus( void *dummy )
{
   if ( !mcuc_busy() )
   {
      union packet p = mcuc_makeStatusPacket();
      mcuc_sendPacket( &p );
   }
}
static void poke( const char *dummy )
{
   puts( "POKE!" );
   union packet p[ 2 ] = { { .header = {
      .type = Type_poke,
      .query = true,
      .length = 1,
   } }, { .data = rand() } };
   p[ 0 ].type = PackType_header;
   mcuc_sendPacket( p );
}

static STATICTEXT *tekst;

int main( int argc, char *argv[] ) {
   ANSA = 0x0000;
   ANSB = 0x0000;
   ANSC = 0x0000;
   ANSD = 0x0000;
   ANSE = 0x0000;
   ANSF = 0x0000;
   ANSG = 0x0000;

   PPSUnLock;
   /* DEBUG UART RX: */
   PPSInput( PPS_U2RX, PPS_RPI37 );
   /* DEBUG UART TX: */
   PPSOutput( PPS_RP22, PPS_U2TX );
   PPSLock;

   struct command commands[] = {
      { "erase", &flashErase },
      { "read", &flashReadTest },
      { "write", &flashWriteTest },
      { "spi", &spiTest },
      { "poke", &poke },
   };
   uart_init( commands, 5 );
   puts( "UART!" );

   // i2c_init( 157, false, 2 ); // 100 kHz
   i2c_init( 37, true, 2 ); // 400 kHz
   touch_init();

   tickTimer_init( 10, 4 );
   // tickTimer_addCallback( &mcucStatus, NULL, 1, 0, -1 );
   // tickTimer_addCallback( &spi, NULL, 0, 80, -1 );
   tickTimer_addCallback( &stillAliveTicker, NULL, 2, 0, -1 );
   // tickTimer_addCallback( &spiTest, NULL, 4, 0, -1 );

   GOLInit();

   InvalidateAll();
   ClearDevice();
   UpdateDisplayNow();

   /* Is this set-up necessary for the M29 flash?: */
#define EPMPCS2_DATA_SETUP_TIME 5
#define EPMPCS2_DATA_WAIT_TIME 80
#define EPMPCS2_DATA_HOLD_TIME 5
   WORD pClockPeriod = (1000000000ul) / GetPeripheralClock();
   if (EPMPCS2_DATA_SETUP_TIME < (pClockPeriod/4))
      PMCS2MDbits.DWAITB = 0;
   else if (EPMPCS2_DATA_SETUP_TIME >= (pClockPeriod/4))
   {
      PMCS2MDbits.DWAITB = (EPMPCS2_DATA_SETUP_TIME / pClockPeriod);
      if ((EPMPCS2_DATA_SETUP_TIME % pClockPeriod) > 0)
         PMCS2MDbits.DWAITB += 1;
   }

   if (EPMPCS2_DATA_WAIT_TIME < ((pClockPeriod*3)/4))
      PMCS2MDbits.DWAITM = 0;
   else if (EPMPCS2_DATA_WAIT_TIME >= ((pClockPeriod*3)/4))
   {
      PMCS2MDbits.DWAITM = (EPMPCS2_DATA_WAIT_TIME / pClockPeriod);
      if ((EPMPCS2_DATA_WAIT_TIME % pClockPeriod) > 0)
         PMCS2MDbits.DWAITM += 1;
   }

   if (EPMPCS2_DATA_HOLD_TIME <= (pClockPeriod/4))
      PMCS2MDbits.DWAITE = 0;
   else if (EPMPCS2_DATA_HOLD_TIME >= (pClockPeriod/4))
   {
      PMCS2MDbits.DWAITE = (EPMPCS2_DATA_HOLD_TIME / pClockPeriod);
      if ((EPMPCS2_DATA_HOLD_TIME % pClockPeriod) > 0)
         PMCS2MDbits.DWAITE += 1;
   }


   if ( m29w320_test() )
      puts( "FEIL!" );
   else
      puts( "riktig!" );

   printf( "Chip size: 2^%d\n", m29w320_size() );

   keyboard_init( 150, L"Testtekst" );

   //mcuc_init();

   /* Initialise LED: */
   TRISFbits.TRISF7 = 0;

   for ( ;; )
   {
      //mcuc_doSPIWork();
      /* Call any callbacks if any timers have expired: */
      tickTimer_handleCallbacks();

      /* Perform any pending drawing operations: */
      GOLDraw();

      GOL_MSG touchMsg;
      /* Get updated touch data: */
      if ( touch_getTouchData( &touchMsg ) )
      {
         /* Finish all pending drawing operations before sending touch data to
          * GOL message handler: */
         while ( !GOLDraw() );
         GOLMsg( &touchMsg );
      }

      uart_runCmdCallbacks();
   }

   return 0;
}

WORD GOLDrawCallback()
{

   return 1;
}

WORD GOLMsgCallback( WORD message, OBJ_HEADER *currentObject, GOL_MSG *
      rawMessage )
{
   if ( !keyboard_msgCallback( message, currentObject, rawMessage ) )
      return 0;

   return 1;
}

WORD ExternalMemoryCallback( IMAGE_EXTERNAL *memory, LONG offset, WORD nCount,
      void *buffer )
{

   return nCount;
}

static void stillAliveTicker( void *dummy )
{
   printf( "TICK %d\n", tickTimer_getTick().s );
   LATFbits.LATF7 ^= 1;
}

static void flashErase( const char *dummy )
{
   puts( "Erasing chip ..." );
   m29w320_eraseChip();
   puts( "Erasing done(?)" );
}

static void flashReadTest( const char *dummy )
{
   puts( "Flash read test ..." );

   int i;
   unsigned char fjas[ 10 ] = {};

   m29w320_readBytesBlocking( 0, fjas, 10 );

   for ( i = 0; i < 10; ++i )
      printf( "%d ", fjas[ i ] );

   puts( "\nRead test done" );
}
static void flashWriteTest( const char *dummy )
{
   puts( "Flash write test ..." );

   unsigned char fjas[] = {1,2,3,4,5,6,7,8,9,10};

   if ( m29w320_writeBytes( 0, fjas, 10 ) )
      printf( "Writing to address %d failed\n", 0 );

   puts( "Write tst done" );
}

static void spiTest( const char *string )
{
   puts( "SPI" );
   union packet p = { .header = { .type = Type_name, .query = true } };
   p.type = PackType_header;
   mcuc_sendPacket( &p );
   puts( "Sent" );
}

void _ISR __attribute__(( no_auto_psv )) _StackError()
{
   puts( "Stack error!" );
   for ( ;; );
}

void _ISR __attribute__(( no_auto_psv )) _AddressError()
{
   puts( "Address error!" );
   for (;;);
}
