/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TCPIPCONFIG_H
#define TCPIPCONFIG_H

#include "GenericTypeDefs.h"
#include "Compiler.h"

/* Enable/include TCP/IP stack components --------------------------------- */
#define STACK_USE_UDP
#define STACK_USE_TCP
#define STACK_USE_SHA1
#define STACK_USE_FATFS
#define STACK_USE_HTTP2_SERVER
#define STACK_USE_SSL_SERVER
#define STACK_USE_IP_ACL
#define STACK_USE_ICMP_SERVER
#define STACK_USE_DHCP_CLIENT
#define STACK_USE_SNTP_CLIENT
/* ------------------------------------------------------------------------ */

/* Miscellaneous stack configuration -------------------------------------- */
#define UDP_USE_TX_CHECKSUM
/* The only need for UDP sockets is for DHCP and SNTP: */
#define MAX_UDP_SOCKETS          4
/* ------------------------------------------------------------------------ */

/* Ethernet controller configuration -------------------------------------- */
/* Use auto-negotiation: */
#define ETH_CFG_LINK             0

#define EMAC_TX_DESCRIPTORS      3
#define EMAC_RX_DESCRIPTORS      7
#define EMAC_RX_BUFF_SIZE        1536
/* ------------------------------------------------------------------------ */

/* HTTP server configuration ---------------------------------------------- */
/* HTTP2 server: */
#define MAX_HTTP_CONNECTIONS     10
#define MAX_HTTP_FILE_HANDLES    13
#define HTTP_DEFAULT_FILE        "index.html"
#define HTTPS_DEFAULT_FILE       "index.html"
/* For buffer overrun protection: */
#define HTTP_DEFAULT_LEN         128
#define HTTP_PORT                80
#define HTTPS_PORT               443
#define HTTP_MIN_CALLBACK_FREE   16
#define HTTP_USE_POST
#define HTTP_USE_AUTHENTICATION
#define HTTP_ROOT_DIR_PATH       "/www"

#if defined __HTTP2_C || defined FORM_H
/* HTTP POST configuration: */
/* Maps file name in URL in POST requests to a an ID, which in turn will map
 * to an appropriate callback function. */
enum POSTFileNameIDs
{
   POSTFileName_network = 0,
   POSTFileName_time,
   POSTFileName_availType,
   POSTFileName_bio,
};
#endif // defined __HTTP2_C || defined FORM_H
#if defined __HTTP2_C
const char *POSTFileIDMapping[] =
{
   [ POSTFileName_network ] = "config/networkPOST.html",
   [ POSTFileName_time ] = "config/timePOST.html",
   [ POSTFileName_availType ] = "config/availTypePOST.html",
   [ POSTFileName_bio ] = "config/bioPOST.html",
};
#endif // __HTTP2_C

#define MAX_SSL_CONNECTIONS      2
/* Maximum number of cached SSL sessions: */
#define MAX_SSL_SESSIONS         2
#define MAX_SSL_BUFFERS          4 // 2 per socket is needed
#define MAX_SSL_HASHES           5 // 2 per socket + 1 extra
#define SSL_RSA_KEY_SIZE         2048
//#define HTTP_NO_AUTH_WITHOUT_SSL
/* ------------------------------------------------------------------------ */

/* SNTP client configuration ---------------------------------------------- */
#define NTP_DEFAULT_SERVER       "0.no.pool.ntp.org"
#define NTP_SERVER_MAX_LEN       30
#define NTP_DEFAULT_QUERY_INTERVAL ( 10ull * 60ull * TICK_SECOND )
/* ------------------------------------------------------------------------ */

/* Configure socket and buffer memory alocation --------------------------- */
/* The following definitions are used to name the various socket types used in
 * the application. Only HTTP is used. */
#define TCP_PURPOSE_HTTP_SERVER  0
/* This definition is needed by a legacy macro in TCP.h; no socket memory
 * initialisation in TCPSocketInitializer is actually needed. */
#define TCP_PURPOSE_DEFAULT      1

#define HTTP_SOCKET_RX_SIZE      1500
#define HTTP_SOCKET_TX_SIZE      1500
/* Memory allocation: */
/* Only PIC RAM is eligible (the LAN8720 does not have any RAM). The size is
 * calculated by adding the size of the RX and TX buffer, and 50, for each
 * socket allocated in TCPSocketInitializer. */
#define TCP_ETH_RAM_SIZE         0
#define TCP_PIC_RAM_SIZE ( MAX_HTTP_CONNECTIONS * ( 50 + HTTP_SOCKET_RX_SIZE \
         + HTTP_SOCKET_TX_SIZE ) )
#define TCP_SPI_RAM_SIZE         0
#define TCP_SPI_RAM_BASE_ADDRESS 0

#if defined(__TCP_C)
ROM struct
{
	BYTE vSocketPurpose;
	BYTE vMemoryMedium;
	WORD wTXBufferSize;
	WORD wRXBufferSize;
} TCPSocketInitializer[] =
{
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
   { TCP_PURPOSE_HTTP_SERVER, TCP_PIC_RAM, HTTP_SOCKET_TX_SIZE, HTTP_SOCKET_RX_SIZE },
};
#endif
/* ------------------------------------------------------------------------ */

//FIXME: move to somewhere else:
//#define FATFS_DEBUG

#endif // TCPIPCONFIG_H
