/* "bio" module
 *
 * Set and retrieve name and title from file
 *
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "bio.h"
#include "fatFS/ff.h"
#include "fatFS/fsUtils.h"
#include <string.h>

static int openBioFile( FIL *file, const char *fileName, unsigned char
      flags );


char *bio_name()
{
   FIL f;
   FRESULT retVal = openBioFile( &f, BIO_NAME_FILE_NAME, FA_READ );
   if ( retVal != FR_OK )
      return NULL;

   char *name = readStringFromFile( &f );
   f_close( &f );
   return name;
}

int bio_setName( const char *name )
{
   if ( !name )
      return -1;

   FIL f;
   FRESULT retVal = openBioFile( &f, BIO_NAME_FILE_NAME, FA_WRITE |
         FA_CREATE_ALWAYS );
   if ( retVal != FR_OK )
      return retVal;

   unsigned int len = strlen( name ) + 1;
   unsigned int numBytesWritten;
   retVal = f_write( &f, name, len, &numBytesWritten );
   if ( retVal != FR_OK )
      return retVal;
   if ( numBytesWritten != len )
      return -2;

   f_close( &f );
   return FR_OK;
}

char *bio_title()
{
   FIL f;
   FRESULT retVal = openBioFile( &f, BIO_TITLE_FILE_NAME, FA_READ );
   if ( retVal != FR_OK )
      return NULL;

   char *title = readStringFromFile( &f );
   f_close( &f );
   return title;
}

int bio_setTitle( const char *title )
{
   if ( !title )
      return -1;

   FIL f;
   FRESULT retVal = openBioFile( &f, BIO_TITLE_FILE_NAME, FA_WRITE |
         FA_CREATE_ALWAYS );
   if ( retVal != FR_OK )
      return retVal;

   unsigned int len = strlen( title ) + 1;
   unsigned int numBytesWritten;
   retVal = f_write( &f, title, len, &numBytesWritten );
   if ( retVal != FR_OK )
      return retVal;
   if ( numBytesWritten != len )
      return -2;

   f_close( &f );
   return FR_OK;
}

static int openBioFile( FIL *file, const char *fileName, unsigned char flags )
{
   FRESULT retVal = f_open( file, fileName, flags );
   if ( retVal == FR_NO_PATH )
   {
      retVal = f_mkdir( BIO_DIR );
      if ( retVal != FR_OK )
         return retVal;

      return f_open( file, fileName, flags );
   }
   else
      return retVal;
}
