/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "settings.h"
#include "timeSettings.h"
#include "netwSettings.h"
#include "fatFS/ff.h"
#include <stdio.h>
#include <TCPIP Stack/TCPIP.h>
#include <TCPIP Stack/StackTsk.h>

void loadSettings()
{
   FRESULT retVal = loadNetworkSettingsFromFile();
   if ( retVal == FR_NO_FILE || retVal == FR_NO_PATH )
      puts( "WARNING: Could not load network settings from file" );
   else if ( retVal != FR_OK )
      printf( "ERROR: Could not load network settings: %d\n", retVal );

   if ( retVal != FR_OK )
      puts( "Using factory network settings" );

   if ( AppConfig.Flags.bIsDHCPEnabled )
      puts( "Using DHCP ..." );
   else
      printf(
            "Static network settings:\n"
            "  Address: %d.%d.%d.%d\n"
            "  Subnet:  %d.%d.%d.%d\n"
            "  Gateway: %d.%d.%d.%d\n"
            "  DNS 1:   %d.%d.%d.%d\n"
            "  DNS 2:   %d.%d.%d.%d\n",
            AppConfig.MyIPAddr.v[0], AppConfig.MyIPAddr.v[1],
            AppConfig.MyIPAddr.v[2], AppConfig.MyIPAddr.v[3],
            AppConfig.MyMask.v[0], AppConfig.MyMask.v[1],
            AppConfig.MyMask.v[2], AppConfig.MyMask.v[3],
            AppConfig.MyGateway.v[0], AppConfig.MyGateway.v[1],
            AppConfig.MyGateway.v[2], AppConfig.MyGateway.v[3],
            AppConfig.PrimaryDNSServer.v[0], AppConfig.PrimaryDNSServer.v[1],
            AppConfig.PrimaryDNSServer.v[2], AppConfig.PrimaryDNSServer.v[3],
            AppConfig.SecondaryDNSServer.v[0],
            AppConfig.SecondaryDNSServer.v[1],
            AppConfig.SecondaryDNSServer.v[2],
            AppConfig.SecondaryDNSServer.v[3] );

   retVal = loadTimeSettingsFromFile();
   if ( retVal != FR_OK )
      printf( "ERROR: Could not load time settings from file" );
}
