/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "timeSettings.h"
#include "fatFS/ff.h"
#include "TCPIPConfig.h"
#include <TCPIP Stack/SNTP.h>
#include "dateTime.h"

struct timeSettings
{
   unsigned long SNTPUpdateInterval;
   int hourOffset;
   char SNTPPoolServer[ NTP_SERVER_MAX_LEN ];
};

int storeTimeSettingsToFile()
{
   FIL f;
   FRESULT retVal;

   retVal = f_open( &f, TIME_SETTINGS_FILE, FA_WRITE | FA_CREATE_ALWAYS );
   /* If the settings sub directory does not exist, try to create it: */
   if ( retVal == FR_NO_PATH )
   {
      retVal = f_mkdir( SETTINGS_DIR );
      if ( retVal != FR_OK )
         return retVal;

      retVal = f_open( &f, TIME_SETTINGS_FILE, FA_WRITE | FA_CREATE_ALWAYS );
   }
   /* If another error is encountered, abort: */
   if ( retVal != FR_OK )
      return retVal;

   struct timeSettings settings =
   {
      .SNTPUpdateInterval = SNTPGetQueryInterval(),
      .hourOffset = getRTCCOffset(),
   };
   strcpy( settings.SNTPPoolServer, SNTPGetPoolServer() );

   unsigned int numBytesWritten;
   retVal = f_write( &f, &settings, sizeof( settings ), &numBytesWritten );

   f_close( &f );

   if ( retVal )
      return retVal;
   if ( numBytesWritten != sizeof( settings ) )
      return -1;

   return FR_OK;
}

int loadTimeSettingsFromFile()
{
   FIL f;
   FRESULT retVal;

   retVal = f_open( &f, TIME_SETTINGS_FILE, FA_READ );
   if ( retVal != FR_OK )
      return retVal;

   struct timeSettings settings;
   unsigned int numBytesRead;
   retVal = f_read( &f, &settings, sizeof( settings ), &numBytesRead );

   f_close( &f );

   if ( retVal )
      return retVal;
   if ( numBytesRead != sizeof( settings ) )
      return -1;

   SNTPSetQueryInterval( settings.SNTPUpdateInterval );
   setRTCCOffset( settings.hourOffset );
   SNTPSetPoolServer( settings.SNTPPoolServer );

   return FR_OK;
}
