/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "dateTime.h"
#include <TCPIP Stack/SNTP.h>
#include <peripheral/rtcc.h>

static int hourOffset = 0;

time_t time( time_t *tod )
{
   if ( tod )
      return *tod = SNTPGetUTCSeconds() + 60 * 60 * hourOffset;

   return SNTPGetUTCSeconds() + 60 * 60 * hourOffset;
}

void initRTCC()
{
   rtccDate d;
   d.wday   = RTCC_DEF_WEEKDAY;
   d.mday   = RTCC_DEF_DAY_OF_MONTH;
   d.mon    = RTCC_DEF_MONTH;
   d.year   = RTCC_DEF_YEAR;

   rtccTime t;
   t.sec    = RTCC_DEF_SEC;
   t.min    = RTCC_DEF_MIN;
   t.hour   = RTCC_DEF_HOUR;

   RtccOpen( t.l, d.l, 0 );

   SNTPSetUpdateCallback( &setRTCCFromEpoch );
}

void setRTCCDateTime( const struct tm *tm )
{
   rtccDate d;
   /* Number of years since 1900 -> BCD from 2000: */
   d.year = decToBCD( tm->tm_year - 100 );
   /* Number of months since January -> month number starting from 1: */
   d.mon = decToBCD( tm->tm_mon + 1 );
   d.mday = decToBCD( tm->tm_mday );
   d.wday = decToBCD( tm->tm_wday );

   rtccTime t;
   t.hour = decToBCD( tm->tm_hour );
   t.min = decToBCD( tm->tm_min );
   t.sec = decToBCD( tm->tm_sec );
   t.rsvd = 0;

   RtccSetTimeDate( t.l, d.l );
}

void setRTCCFromEpoch( unsigned long epoch )
{
   if ( epoch <= SNTP_REASONABLE_EPOCH )
      return;

   epoch += 60 * 60 * hourOffset;

   struct tm *ltime = localtime( (time_t *)&epoch );
   setRTCCDateTime( ltime );
}

struct tm getRTCCDateTime()
{
   rtccDate d;
   rtccTime t;
   RtccGetTimeDate( &t, &d );

   struct tm tm;

   /* BCD from 200 -> number of years since 1900: */
   tm.tm_year = BCDToDec( d.year ) + 100;
   /* Month number starting from 1 -> number of months since January: */
   tm.tm_mon = BCDToDec( d.mon - 1);
   tm.tm_mday = BCDToDec( d.mday );
   tm.tm_wday = BCDToDec( d.wday );

   tm.tm_hour = BCDToDec( t.hour );
   tm.tm_min = BCDToDec( t.min );
   tm.tm_sec = BCDToDec( t.sec );

   return tm;
}

void setRTCCOffset( int offset )
{
   if ( hourOffset == offset )
      return;

   hourOffset = offset;
   SNTPForceUpdate();
}

int getRTCCOffset()
{
   return hourOffset;
}
