/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DATETIME_H
#define DATETIME_H

#include <sys/time.h>
#include <time.h>
#include <stdbool.h>

#define RTCC_DEF_HOUR 18
#define RTCC_DEF_MIN 6
#define RTCC_DEF_SEC 1
#define RTCC_DEF_YEAR 13
#define RTCC_DEF_MONTH 4
#define RTCC_DEF_DAY_OF_MONTH 1
#define RTCC_DEF_WEEKDAY 1

#define BCDToDec( bcd ) ( 10 * ( (bcd) >> 4 ) + ( (bcd) & 0x0f ) )
#define decToBCD( dec ) ( ( (dec) / 10 ) << 4 | ( (dec) % 10 ) )

/**
 * \brief A minimum epoch timestamp that will work as a sanity check
 *
 * When a timestamp is acquired using updateRTCCwithSNTP(), it must be larger
 * than SNTP_REASONABLE_EPOCH in order to not be discarded.
 */
#define SNTP_REASONABLE_EPOCH 1356994800

time_t time( time_t *tod );

void initRTCC();

void setRTCCDateTime( const struct tm *tm );

void setRTCCFromEpoch( unsigned long epoch );

struct tm getRTCCDateTime();

void setRTCCOffset( int offset );

int getRTCCOffset();

#define timeIsReasonable() ( time( NULL ) > SNTP_REASONABLE_EPOCH )

#endif // DATETIME_H
