/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "ipACL.h"
#include <string.h>

static struct ipACLRuleSet m_ruleSets[ IPACL_MAX_RULESETS ];

void ipACLSetRuleSets( const struct ipACLRuleSet *ruleSets, int numSets )
{
   if ( !ruleSets )
      return;

   size_t i;
   for ( i = 0; i < numSets && i < IPACL_MAX_RULESETS; ++ i )
      m_ruleSets[ i ] = ruleSets[ i ];
}

void ipACLGetRuleSets( struct ipACLRuleSet *ruleSets, int numSets )
{
   size_t i;
   for ( i = 0; i < numSets && i < IPACL_MAX_RULESETS; ++ i )
      ruleSets[ i ] = m_ruleSets[ i ];
}

bool ipACLAllowConn( WORD protocol, IP_ADDR sourceAddr, WORD destPort )
{
   size_t i;
   struct ipACLRuleSet *ruleSet = NULL;

   //printf( "Checking proto %d, addr %d.%d.%d.%d, port %d\n: ", protocol,
   //      sourceAddr.v[0], sourceAddr.v[1], sourceAddr.v[2], sourceAddr.v[3],
   //      destPort );

   /* Find a rule set that matches the protocol and destination port: */
   for ( i = 0; i < IPACL_MAX_RULESETS; ++ i )
      if ( ( m_ruleSets[ i ].protocol & protocol ) && m_ruleSets[ i ].port
            == destPort )
      {
         ruleSet = &m_ruleSets[ i ];
         break;
      }

   /* If there are no rule sets for the given protocol and destination port,
    * allow: */
   if ( !ruleSet )
      return true;

   for ( i = 0; i < IPACL_MAX_RULES_PER_SET; ++ i )
   {
      struct ipACLRule *r = &ruleSet->rules[ i ];

      /* For efficiency, only check rules up to the first disabled rule (it is
       * the user's resposibility to keep enabled rules at the beginning of
       * the array): */
      if ( !r->ruleEnabled )
         break;

      unsigned long sourceAddrVal = swapl( sourceAddr.Val );
      bool withinRange = sourceAddrVal >= swapl( r->srcAddrRangeStart.Val ) &&
         sourceAddrVal <= swapl( r->srcAddrRangeEnd.Val );

      if ( withinRange )
         return r->pass;
   }

   if ( ruleSet->defaultPass )
      return true;
   else
      return false;
}
