/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef IPACL_H
#define IPACL_H

#include <stdbool.h>
#include <TCPIP Stack/TCPIP.h>

#define IPACL_MAX_RULES_PER_SET 10
#define IPACL_MAX_RULESETS 1

struct ipACLRule {
   IP_ADDR srcAddrRangeStart;
   IP_ADDR srcAddrRangeEnd;
   bool pass;
   bool ruleEnabled;
};

enum protocol {
   Proto_UDP   = 0x01,
   Proto_TCP   = 0x02,
};

struct ipACLRuleSet {
   struct ipACLRule rules[ IPACL_MAX_RULES_PER_SET ];
   WORD protocol;
   WORD port;
   bool defaultPass;
};

void ipACLSetRuleSets( const struct ipACLRuleSet *ruleSets, int numSets );

void ipACLGetRuleSets( struct ipACLRuleSet *ruleSets, int numSets );

/*
 * \brief Check if a connection should be allowed from the given IP address to
 * the given destination port, using the given protocol(s)
 *
 * If there is a rule set that matches the given protocol and port, all of
 * the enabled rules in the set will be checked against the IP address. If no
 * rules match, the ipACLRuleSet::defaultPass setting will determine whether
 * the connection should be allowed or not. Only rules up to the first
 * disabled rule will be checked, so make sure that all enabled rules are
 * placed in start of the ipACLRuleSet::rules array.
 *
 * Once the IP address is matched against the range in a rule, this function
 * returns true or false depending on the rule's ipACLRule::pass settings.
 *
 * \param protocol the protocol used, either Proto_UDP, Proto_TCP or both
 * \param sourceAddr the source IP address
 * \param destPort the destination port
 */
bool ipACLAllowConn( WORD protocol, IP_ADDR sourceAddr, WORD destPort );

#endif // IPACL_H
