/* 
 * Copyright (C) 2013 Andreas Misje
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "led.h"
#include "tickTimer.h"

static void reverseDHCPLEDState( void *dummyArg );
static void reverseUSR2LEDState( void *dummyArg );

void led_init()
{
   /* Set LED pins to output and turn the LEDs off: */

   /* DHCP LED: */
   TRISGbits.TRISG14 = 0;
   LATGbits.LATG14 = 1;

   /* USR_LED2: */
   TRISGbits.TRISG12 = 0;
   LATGbits.LATG12 = 1;
}

void led_flashDHCP( unsigned int rateMs, int numFlashes )
{
   reverseDHCPLEDState( NULL );
   tickTimer_addCallback( &reverseDHCPLEDState, NULL, rateMs,
         numFlashes - 1 );
}

void led_flashUSR2( unsigned int rateMs, int numFlashes )
{
   reverseUSR2LEDState( NULL );
   tickTimer_addCallback( &reverseUSR2LEDState, NULL, rateMs,
         numFlashes - 1 );
}


static void reverseDHCPLEDState( void *dummyArg )
{
   LATGbits.LATG14 ^= 1;
}

static void reverseUSR2LEDState( void *dummyArg )
{
   LATGbits.LATG12 ^= 1;
}
